/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.engine.performance.provider;

import java.util.logging.Level;

import javax.jbi.component.ComponentContext;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessageExchangeFactory;
import javax.jbi.messaging.NormalizedMessage;

import org.objectweb.petals.component.common.util.SourceHelper;

/**
 * @author alouis - EBM WebSourcing
 * 
 */
public class AcceptAndSendTask implements Runnable {

    private static String IN_ONLY = "http://www.w3.org/2004/08/wsdl/in-only";

    protected Receiver receiver;

    protected ComponentContext context;

    protected DeliveryChannel channel;

    protected MessageExchangeFactory factory;

    /**
     * Creates a new instance of AcceptandSend thread Task
     * 
     * @param monitoring
     * @param startSignal
     * @param doneSignal
     */
    public AcceptAndSendTask(Receiver receiver) {
        this.receiver = receiver;
        context = receiver.context;
        channel = receiver.channel;
        factory = channel.createExchangeFactory();
    }

    /**
     * @see java.lang.Thread#run()
     */
    public void run() {

        MessageExchange exchange = null;

        try {
            if (receiver.acceptTimeout > 0) {
                exchange = channel.accept(receiver.acceptTimeout);
            } else {
                exchange = channel.accept();
            }
            if (exchange == null) {
                receiver.incNotAcceptedMessageCount();
            } else {

                // Increment the counter
                receiver.incAcceptedMessageCount();
                
                // Trace the tracked messages
                if (exchange.getProperty("tracked") != null
                    && ((String) exchange.getProperty("tracked"))
                        .equals("true")) {
                    String id = exchange.getExchangeId();
                    receiver.acceptMarker.write(id + "\t"
                        + System.currentTimeMillis() + "\n");
                }

                NormalizedMessage in = exchange.getMessage("IN");

                // Process received message
                if (receiver.processInContent) {
                    SourceHelper.createString(in.getContent());
                }

                // DONE or message OUT response
                if (exchange.getPattern().toString().equals(IN_ONLY)) {
                    exchange.setStatus(ExchangeStatus.DONE);
                } else {
                    exchange.setMessage(in, "OUT");
                }

                // Send the response
                try {
                    channel.send(exchange);
                    receiver.incSentResponseCount();
                } catch (Exception e) {
                    receiver.incNotSentResponseCount();
                }
            }
        } catch (Exception e) {
            receiver.logger.log(Level.SEVERE, e.getMessage(), e);
        }

    }
}
